﻿define(['qvangular',
    'text!QvSSASConnector.webroot/connectdialog.ng.html',
    'css!QvSSASConnector.webroot/connectdialog.css'
], function (qvangular, template) {
    return {
        template: template,
        controller: ['$scope', 'input', function ($scope, input) {

            function getParameterUpdateScope(parameterName, scopeMember) {
                input.serverside.sendJsonRequest("getConnectionParameter", parameterName).then(function (info) {
                    console.log("info", info, scopeMember);
                    $scope[scopeMember] = info.qMessage;
                });
            }

            function getParameter(parameterName) {
                return input.serverside.sendJsonRequest("getConnectionParameter", parameterName);
            }


            function init() {
                $scope.isEdit = input.editMode;
                $scope.id = input.instanceId;
                $scope.titleText = $scope.isEdit ? "Change Cube Connector" : "Add Cube Connector";
                $scope.saveButtonText = $scope.isEdit ? "Save changes" : "Create";

                //populate authentication dropdown
                $scope.authenticationTypeOptions = [
                    //{
                    //    id: "AZURE_TOKEN",
                    //    description: "Both"
                    //},
                    {
                        id: "AZURE_USER_PASSWORD",
                        description: "Azure username & password"
                    },
                    {
                        id: "AZURE_INTEGRATED_SECURITY",
                        description: "Azure integrated security"
                    },
                    {
                        id: "AZURE_INTERACTIVE",
                        description: "Azure interactive (Deskop only!)"
                    },
                    {
                        id: "WINDOWS_USER_PASSWORD",
                        description: "Windows username & password"
                    },
                    {
                        id: "WINDOWS_INTEGRATED_SECURITY",
                        description: "Windows integrated security"
                    }
                ]

                //connection info
                $scope.host = "";
                $scope.name = "";
                $scope.username = "";
                $scope.password = "";
                $scope.authenticationType = getAuthenticationTypeFromId("WINDOWS_USER_PASSWORD");
                $scope.Port = "2383";
                $scope.provider = "SSASConnector.exe"; // Connector filename
                $scope.connectionInfo = "";
                $scope.connectionSuccessful = false;
                $scope.connectionString = createCustomConnectionString($scope.provider, $scope.host);

                //gui control
                $scope.testingInProgress = false;

                //get server info
                input.serverside.sendJsonRequest("getInfo").then(function (info) {
                    $scope.info = info.qMessage;
                });

                //populate data if in edit
                if ($scope.isEdit) {
                    input.serverside.getConnection($scope.id).then(function (result) {
                        console.log("result", result);
                        $scope.name = result.qConnection.qName;
                        $scope.authenticationType
                    });
                    getParameterUpdateScope("host", "host");
                    getParameterUpdateScope("userid", "username");

                    //authentication type
                    getParameter("authenticationType").then(function (authenticationType) {
                        console.log("authenticationType", authenticationType);
                        if (authenticationType.qMessage.indexOf("Not found") == 0) { //check if not is returned
                            //do nothing keep default
                        } else {
                            $scope.authenticationType = getAuthenticationTypeFromId(authenticationType.qMessage);                          
                        }
                    });

                }


            }

            //debug info
            console.log('org connect dialog', Date.now(), input);

            //functions
            function validScopeInput(scopeVariableArray) {
                for (index = 0; index < scopeVariableArray.length; ++index) {
                    scopeVariable = scopeVariableArray[index];
                    //console.log("test", scopeVariable.property, $scope[scopeVariable.property], $scope[scopeVariable.property].length, !!$scope[scopeVariable.property], $scope[scopeVariable.property].length > 0, !!scopeVariable.regex, scopeVariable.regex);
                    //json used to test objects
                    if (!!$scope[scopeVariable.property] && ($scope[scopeVariable.property].length > 0 || JSON.stringify($scope[scopeVariable.property]).length > 2)) {
                        //has regex?
                        if (!!scopeVariable.regex) {
                            if (!scopeVariable.regex.test($scope[scopeVariable.property])) {
                                return { message: "invalid format " + scopeVariable.regexError + "", valid: false };
                            }
                        }
                        //ok
                    } else {
                        return { message: "missing " + scopeVariable.error, valid: false };
                    }
                }
                return { message: "Input ok", valid: true };
            }

            function getAuthenticationTypeFromId(id) {
                return $scope.authenticationTypeOptions.filter(function (type) {
                    return type.id === id;
                })[0];
            }


            /* Event handlers */

            $scope.onOKClicked = function () {
                if ($scope.isEdit) {
                    var overrideCredentials = ($scope.username !== "" && $scope.password !== "");
                    //update connection string
                    $scope.connectionString = createCustomConnectionString($scope.provider, $scope.host, $scope.authenticationType.id);
                    //update in qlik
                    input.serverside.modifyConnection($scope.id,
                        $scope.name,
                        $scope.connectionString,
                        $scope.provider,
                        overrideCredentials,
                        $scope.username,
                        $scope.password).then(function (result) {
                            if (result) {
                                $scope.destroyComponent();
                            }
                        });
                } else {
                    //create connection string
                    $scope.connectionString = createCustomConnectionString($scope.provider, $scope.host, $scope.authenticationType.id);
                    //create in qlik
                    input.serverside.createNewConnection($scope.name, $scope.connectionString, $scope.username, $scope.password);
                    $scope.destroyComponent();
                }
            };

            $scope.onTestConnectionClicked = function () {

                $scope.connectionInfo = "";

                var scopeVariables = [
                    { property: "name", error: "connection name" },
                    { property: "host", error: "host" },
                    { property: "Port", error: "port" },
                    { property: "authenticationType", error: "authentication type" }
                ];

                //check for username
                if (!!$scope.authenticationType && $scope.usesUsernameAndPassword()) {
                    scopeVariables.push({ property: "username", error: "username" })

                    //check for password if first time
                    if (!$scope.isEdit) {
                        scopeVariables.push({ property: "password", error: "password" });
                    }

                }



                var inputValidation = validScopeInput(scopeVariables);

                if (!inputValidation.valid) {
                    $scope.connectionInfo = inputValidation.message;
                    $scope.connectionSuccessful = false;
                    return;
                }

                $scope.testingInProgress = true;
                input.serverside.sendJsonRequest("testConnection", $scope.username, $scope.password, $scope.host, $scope.authenticationType.id).then(function (info) {
                    console.log("Info", info);
                    $scope.connectionInfo = info.message;
                    $scope.connectionSuccessful = info.access;
                    $scope.testingInProgress = false;
                    console.log($scope.connectionSuccessful);
                });
            };

            //gui control

            $scope.isOkEnabled = function () {
                return !!$scope.name && $scope.name.length > 0 && $scope.connectionSuccessful && !$scope.testingInProgress;
            };

            $scope.usesUsernameAndPassword = function () {
                return ($scope.authenticationType.id === "AZURE_USER_PASSWORD" || $scope.authenticationType.id === "WINDOWS_USER_PASSWORD");
            };

            $scope.onEscape = $scope.onCancelClicked = function () {
                $scope.destroyComponent();
            };



			
			/* Helper functions */


            function createCustomConnectionString(filename, host, authenticationType) {
                //var connectionString = "CUSTOM CONNECT TO " + "\"provider=" + filename + ";host=" + host + ";database=" + database + ";catalog=" + catalog + ";\"";
                var connectionString = "CUSTOM CONNECT TO " + "\"provider=" + filename + ";host=" + host + ";authenticationType=" + authenticationType + ";\"";
                console.log("string", connectionString);
                return connectionString;
            }

            // Help Menu

            //$scope.helpMenuButton = {};

            //function createHelpMenu() {
            //    console.log("connector service", connectorConnectHelpMenuService);
            //    if (connectorConnectHelpMenuService != undefined) {
            //        $scope.helpMenuButton = connectorConnectHelpMenuService.CreateHelpMenu(function () {
            //            var isLocalHost = window.location.hostname === "localhost";
            //            return qvangular.promise({ supportsLicensing: true && isLocalHost });
            //        },
            //            function (menuAction) {
            //                if (menuAction === 'help') {
            //                    $scope.showHelp();
            //                }
            //                else if (menuAction === 'about') {
            //                    $scope.showInfo('info');
            //                }
            //                else if (menuAction === 'license') {
            //                    input.serverside.sendJsonRequest("updateLicense");
            //                }
            //            }
            //        );
            //    } else {
            //        console.log("no help menu service");
            //    }
            //}

            //createHelpMenu();

			init();
		}]
	};
} );