define(['qvangular',
    'underscore',
    'text!QvSSASConnector.webroot/selectdialog.ng.html',
    'css!QvSSASConnector.webroot/selectdialogstyle.css'
], function (qvangular, underscore, template) {
    return {
        template: template,
        controller: ['$scope', 'serverside', function ($scope, serverside) {
            var ssasDialogContentProvider = {
                getConnectionInfo: function () {
                    return qvangular.promise({
                        dbusage: false,
                        ownerusage: false,
                        dbseparator: '.',
                        ownerseparator: '.',
                        specialchars: '! "$&\'()*+,-/:;<>`{}~[]',
                        quotesuffix: '"',
                        quoteprefix: '"',
                        dbfirst: true,
                        keywords: []
                    });
                },
                getCatalogs: function () {
                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getCatalogs").then(function (response) {
                        $scope.loadInProgress = false;

                        //console.log("database response", response);
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No catalog was returned";
                            console.log($scope.errorMessage);
                            return [{ "name": "No catalog was returned","id":"nocatalog" }];
                        }
                        return response.catalogs;
                    });
                },
                getCubes: function (cubeName) {
                    console.log("cubes", cubeName);
                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getCubes", cubeName).then(function (response) {
                        $scope.loadInProgress = false;

                        //console.log("cubes response", response)
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No cubes was returned";
                            console.log($scope.errorMessage);
                            return [{ "name": "No cubes was returned", "id":"nocube" }];
                        }
                        return response.cubes;
                    });
                },
                //getTables: function (catalogName, cubeName) {
                //    return serverside.sendJsonRequest("getTables", catalogName, cubeName).then(function (response) {
                //        return response.qTables;
                //    });
                //},
                getCubeContainer: function (catalogName, cubeName) {
                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getCubeContainer", catalogName, cubeName).then(function (response) {
                        $scope.loadInProgress = false;
                        return response.cubeContainer;
                    });
                },
                getMembers: function (catalogName, cubeName, dimensionId) {
                    return serverside.sendJsonRequest("getMembers", catalogName, cubeName, dimensionId).then(function (response) {
                        console.log("members response", response);
                        return response.dimensionMembers;
                    });
                },
                getFields: function (catalogName, cubeName, tableName) {
                    return serverside.sendJsonRequest("getFields", catalogName, cubeName, tableName).then(function (response) {
                        return response.qFields;
                    });
                },
                getPreview: function ( /*databaseName, ownerName, tableName*/) {
                    return qvangular.promise([]);
                }
            };

            //Filters

            //init of scope
            function init() {
                //Model variables 
                $scope.catalogs = [];
                $scope.cubes = [];
                $scope.cubeContent = [];
                $scope.selectedCubeContent = [];
                $scope.errorMessage = "";
                $scope.loadInProgress = false;

                //View control variables
                $scope.dataview = false;
                $scope.helpview = false;
                $scope.metadataview = true;

                $scope.currentCatalog = "Select db";
                $scope.currentCube = "Select catalog";

                //query settings
                $scope.dimensionIncludeEmpty = true;
                $scope.measureIncludeEmpty = true;

                //contentSettingsDimension
                $scope.contentSettings = {
                    dimension: [
                        {
                            id: "1",
                            description: "Both"
                        },
                        {
                            id: "2",
                            description: "Caption"
                        },
                        {
                            id: "3",
                            description: "Unique Name"
                        },
                        {
                            id: "4",
                            description: "Filter only"
                        }
                    ]
                };

                //get initial catalogs
                ssasDialogContentProvider.getCatalogs().then(function (catalogs) {
                    console.log("catalog", catalogs);
                    $scope.catalogs = catalogs;
                });

            }

            //updates
            $scope.getCubes = function (catalog) {

                //reset the cubes
                $scope.cubes = null;

                ssasDialogContentProvider.getCubes(catalog).then(function (cubes) {
                    console.log("cubes", cubes);
                    $scope.cubes = cubes;
                })
            }

            $scope.getCubeContent = function (catalog, cube) {

                //reset cube content
                $scope.cubeContent = null;

                //check if tables i already loaded
                //if (($scope.cubeContent != undefined && $scope.cubeContent.length > 0)) {
                //    return;
                //}

                // set wait indicator
                $scope.gettingTables = true;

                ssasDialogContentProvider.getCubeContainer(catalog, cube).then(function (cubeContainer) {
                    //console.log("getCubeContainer", cubeContainer);

                    var content = [];

                    var KPITree = KPIsToTree(cubeContainer.kPIs);

                    content.push({ id: "MeasureRoot", name: "Measures", contentChildren: cubeContainer.measures });
                    content.push({ id: "KPIRoot", name: "KPIs", contentChildren: KPITree });
                    content.push({ id: "DimensionRoot", name: "Dimensions", contentChildren: cubeContainer.dimensions });

                    //console.log("cubeContent", content);


                    $scope.cubeContent = content;
                    $scope.gettingTables = false;
                });

                return;
            }

            $scope.getMembers = function (catalog, cube, dimensionId) {

                if (!!$scope.selectedCubeContent && $scope.selectedCubeContent.length > 0) {
                    var dimension = $scope.selectedCubeContent.find(function (content) {
                        return content.id == dimensionId;
                    });

                    //console.log("dimension", dimension);

                    //initial placeholder
                    dimension.members = [
                        {
                            id: "1",
                            name: "Loading members"
                        }
                    ];

                    //load from db
                    ssasDialogContentProvider.getMembers(catalog, cube, dimensionId).then(function (members){
                        console.log("got members", members)

                        dimension.members = members;
                    });
                }

            }

            //console.log('org select dialog', Date.now(), $scope);

            //Internal Functions

            function KPIsToTree(KPIs) {
                var KPItree = [];

                //console.log("KPIsToTree KPIs", KPIs);

                KPIs.forEach(function (KPIInput) {

                    var KPI = KPIInput;

                    //console.log("KPI", KPI);

                    KPI.contentChildren = [
                        {
                            id: KPI.kpiGoalExpression,
                            isLeaf: true,
                            name: KPI.name + " Goal",
                            type: "KPI"
                        },
                        {
                            id: KPI.kpiStatusExpression,
                            isLeaf: true,
                            name: KPI.name + " Status",
                            type: "KPI"
                        },
                        {
                            id: KPI.kpiValueExpression,
                            isLeaf: true,
                            name: KPI.name + " Value",
                            type: "KPI"
                        },
                    ];

                    KPI.isLeaf = false;

                    KPItree.push(KPI);

                   // console.log("KPIsToTree", KPItree);

                });

                //console.log("KPIsToTree", KPItree);

                return KPItree;

            }

            // This accomodates the recently changed QlikView API contract
            // without having to make changes to the contract for the connect dialog.
            ///// see C:\Users\micha\AppData\Local\Programs\Common Files\Qlik\Custom Data\QvOdbcConnectorPackage\web\selectdialog.js ///

            function indentLine(level) {
                var indent = "";
                for (var i = 0; i < level; i++) {
                    indent += "\t";
                }
                return indent;
            }

            function filterSelected(list) {
                return underscore.filter(list, function (x) {
                    return x.checked;
                });
            }

            function generateUniqueColumnName(usedNames, name) {
                if (usedNames.indexOf(name) == -1) {
                    //Name not used
                    return name;
                }

                var suggestedName = "";
                var i = 1;

                //find a name
                do {                    
                    if (name.slice(-1) != "]") {
                        var suggestedName = name + " " + i;
                    } else {
                        var suggestedName = name.slice(0, -1) + " " + i + "]";
                    }
                    i++;
                    console.log("Trying name", suggestedName);
                }
                //try differnt names, terminate after 10000s tries to avoid locking the GUI
                while (usedNames.indexOf(suggestedName) != -1 || i > 10000);

                console.log("Generated suggested name  ", suggestedName, " for ", name);

                return suggestedName;
            }

            function createDimensionMemberSelect(content, dimensionProperty, usedNames, suffix) {

                var name = content.name.slice(0, -1) + suffix + "]";

                name = generateUniqueColumnName(usedNames, name);

                usedNames.push(name);

                var listName = indentLine(2) + content.id + dimensionProperty + " as " + name+",\r\n";
                return listName
            }

            function reduceToBaseDimensions(dimensions) {
                var lowestLevelFilteredDimension = dimensions.reduce(function (foundDimensions, currentDimension) {
                    //console.log("currentDimension", currentDimension);

                    //var baseDimension = dimension.id.substring(0, dimension.id.lastIndexOf("."));

                    //check if a prevoius dimension added to the collection with same base dimension
                    var index = foundDimensions.findIndex(
                        function (baseDimension) {
                            return function (element) {
                                console.log("find", element, baseDimension)
                                return element.baseDimension == baseDimension;
                            }
                        }(currentDimension.baseDimension)
                    );

                    if (index != -1) {
                        
                        var previousDimension = foundDimensions[index];

                        //console.log("found baseDimension", previousDimension);

                        //check if on a lower level
                        if (currentDimension.dimensionLevel > previousDimension.dimensionLevel) {
                            //console.log("found new level", currentDimension, previousDimension)

                            ////replace with new current dimension
                            //currentDimension.baseDimension = baseDimension;

                            foundDimensions[index] = currentDimension;
                        } 

                        //check is current dimension has a lower config
                        if (currentDimension.selectConfig.id < previousDimension.baseConfig) {
                            //console.log("found lower config", currentDimension, previousDimension);
                            foundDimensions[index].baseConfig = currentDimension.baseConfig;
                        }
                    }
                    else {
                        //console.log("adding new base dimension ", currentDimension)

                        //add current dimenison to collection
                        //currentDimension.baseDimension = baseDimension;
                        currentDimension.baseConfig = currentDimension.selectConfig.id;

                        foundDimensions.push(currentDimension);
                    }
                    return foundDimensions;
                }, []);
                return lowestLevelFilteredDimension;
            }


            function createFilteredFrom(filteredDimensions, cubeName, baseDimensions) {
                //build filtered from using sub queries

                //var baseCubeFrom = "FROM [" + cubeName + "]";
                var from = indentLine(1) + "FROM [" + cubeName + "]";
                var where = "";



                console.log("Applying filter to " + filteredDimensions.length + " dimension");
                console.log("Filtering on dimesions:", filteredDimensions, " with basedimensions:", baseDimensions);

                filteredDimensions.forEach(function (dimension) {  

                    //find the dimensions baseDimension
                    var baseDimension = baseDimensions.find(function (baseDimension) {
                        return function (element) {
                            return element.baseDimension == baseDimension;
                        };
                    }(dimension.baseDimension));

                    console.log("Filter", dimension, baseDimension);

                    var members = "";

                    dimension.selectedMembers.forEach(function (member){
                        //build list of members
                        members += indentLine(2) + member.id + ",\r\n";

                    });

                    if (!baseDimension) {
                        console.error("Missing base dimension", dimension);
                    }

                    //check is the is a pure filter in where
                    if (!!baseDimension && baseDimension == 4) {
                        //Add to where
                        console.log("found pure filter, with length: ", dimension.selectedMembers, dimension.selectedMembers.length);

                        if (dimension.selectedMembers.length == 1) {
                            where += indentLine(2) + dimension.selectedMembers[0].id + ",\r\n";
                        } else if (dimension.selectedMembers.length > 1) {
                            
                            where += indentLine(2) + dimension.baseDimension + ".currentmember" + ",\r\n";
                        } else {
                            console.log("Missing members");
                        }

                    } 

                    //incapsulate memberlist
                    members = "{\r\n" + members.slice(0, -3) + "\r\n" + indentLine(2) + "}";

                    from = indentLine(1) + "FROM ( SELECT ( " + members + " ) ON COLUMNS\r\n" + from + ")";

                });

                if (where.trim().length > 0) {
                    //incapsulate where
                    where = indentLine(1) + "WHERE (\r\n" + where.slice(0, -3) + "\r\n" + indentLine(1) + ")";
                }

                from = from + "\r\n" + where;

                //console.log("From ", from);

                return from;

            }

            function getCurrentScript(selectedCubeContent, cubeName, catalogName, includeNonEmptyDimension, includeNonEmptyMeasure) {
                var script = "";

                var qlikColumnDefinition = "";
                var dimensionQuery = "";
                var measureQuery = "";
                var loadMemberCaption = false;
                var loadMemberUniqueName = false;
                var filteredDimensions = [];
                var selectedDimensions = [];
                var selectedMeasures = [];
                var selectedKPIs = [];
                var usedNames = [];


                //Split selected content into types for processing
                selectedCubeContent.forEach(function (content){

                    //console.log("Content info", content);

                    if (content.type == "Dimension") {

                        content.baseDimension = content.id.substring(0, content.id.lastIndexOf("."));
                        selectedDimensions.push(content);

                    } //else { console.log("not dimension", content.type, content.type == "Dimension") }

                    if (content.type == "Measure") {
                        selectedMeasures.push(content);

                    } //else { console.log("not measure", content.type, content.type == "Measure") }

                    if (content.type == "KPI") {
                        selectedKPIs.push(content);

                    } //else { console.log("not KPI", content.type, content.type == "KPI") }

                });

                //build dimension query
                var baseDimensions = reduceToBaseDimensions(selectedDimensions);
                baseDimensions.forEach(function (dimension) {
                    //add all members to id
                    var selectName = dimension.id + ".ALLMEMBERS";

                    //add to measures - if the dimension isn't purely used as filter
                    if (dimension.baseConfig != 4) {
                        dimensionQuery += indentLine(2) + selectName + " *\r\n";
                    }
                });

                //Build dimensions definitions
                selectedDimensions.forEach(function (dimension) {


                    //console.log("content.selectConfig", dimension.selectConfig.id);


                    //added to filtered is seletedMembers is set
                    if (!!dimension.selectedMembers && dimension.selectedMembers.length > 0) {
                        filteredDimensions.push(dimension);
                    }

                    //add dimension property
                    if (dimension.selectConfig.id == 1) {                        
                        qlikColumnDefinition += createDimensionMemberSelect(dimension, ".[MEMBER_CAPTION]", usedNames, "");
                        qlikColumnDefinition += createDimensionMemberSelect(dimension, ".[MEMBER_UNIQUE_NAME]", usedNames, " Unique Name");
                        loadMemberCaption = true;
                        loadMemberUniqueName = true;

                    } else if (dimension.selectConfig.id == 2) {
                        qlikColumnDefinition += createDimensionMemberSelect(dimension, ".[MEMBER_CAPTION]", usedNames, "");
                        loadMemberCaption = true;

                    } else if (dimension.selectConfig.id == 3) {
                        qlikColumnDefinition += createDimensionMemberSelect(dimension, ".[MEMBER_UNIQUE_NAME]", usedNames, "");
                        loadMemberUniqueName = true;
                    } else if (dimension.selectConfig.id == 4) {
                        //Dimension is use for filter
                    } else {
                        qlikColumnDefinition += createDimensionMemberSelect(dimension, ".[MEMBER_CAPTION]", usedNames, "");
                        loadMemberCaption = true;
                    }
                });

                //Build measures
                selectedMeasures.forEach(function (measure){
                    measureQuery += indentLine(2) + measure.id + ",\r\n";
                    qlikColumnDefinition += indentLine(2) + measure.id + " as [" + measure.name + "],\r\n";
                });

                //Build KPIs
                selectedKPIs.forEach(function (KPI) {
                    measureQuery += indentLine(2) + KPI.id + ",\r\n";
                    qlikColumnDefinition += indentLine(2) + KPI.id + " as [" + KPI.name + "],\r\n";
                });
                
                //set dimensions properties
                var dimensionProperty = "";

                if (loadMemberCaption && loadMemberUniqueName) {
                    dimensionProperty = "MEMBER_CAPTION, MEMBER_UNIQUE_NAME";
                } else if (loadMemberUniqueName) {
                    dimensionProperty = "MEMBER_UNIQUE_NAME";
                } else {
                    dimensionProperty = "MEMBER_CAPTION";
                }

                //set empty control for dimension
                var dimensionNonEmpty = "NON EMPTY {\r\n";

                if (includeNonEmptyDimension) {
                    dimensionNonEmpty = "{\r\n";
                }

                //set empty control for measure
                var measureNonEmpty = "NON EMPTY {\r\n";

                if (includeNonEmptyMeasure) {
                    measureNonEmpty = "{\r\n";
                }


                script += indentLine(0) + "["+ cubeName + "]:\r\n";
                script += indentLine(0) + "SELECT\r\n";
                script += indentLine(1) + "LOAD\r\n";
                script += indentLine(0) + qlikColumnDefinition.slice(0, -3) + "\r\n"; 
                script += indentLine(1) + "MDX\r\n";
                script += indentLine(1) + "SELECT\r\n";
                script += indentLine(1) + measureNonEmpty;
                script += indentLine(0) + measureQuery.slice(0, -3) + "\r\n";
                script += indentLine(1) + "} ON COLUMNS,\r\n";
                script += indentLine(1) + dimensionNonEmpty;
                script += indentLine(0) + dimensionQuery.slice(0, -3) + "\r\n";
                script += indentLine(1) + "} DIMENSION PROPERTIES " + dimensionProperty + " ON ROWS\r\n";
                script += indentLine(0) + createFilteredFrom(filteredDimensions, cubeName, baseDimensions) + "\r\n";
                script += indentLine(1) + "USE [" + catalogName + "]";
                script += indentLine(0) + ";";


                //console.log("CubeContent", selectedCubeContent, script);

                script += "\r\n\r\n";

                return script.trim();
            }

              ///// Event handlers //////


            //handle Tree view expansion
            $scope.onExpanded = function ($event) {
                var targetScope = $($event.target).scope();

                var contentNode = targetScope.$parent.contentNode;

                contentNode.isExpanded = !contentNode.isExpanded;

            };

            //handle Tree view checkboxes
            $scope.onCubeContentCheckboxClick = function (contentNode) {
                //var list = node;

                var updateSelectedContent = function (contentNode) {
                    //TODO might need to be done for all children
                    if (contentNode.checked && contentNode.isLeaf) {
                        //add to selected content

                        var index = $scope.selectedCubeContent.findIndex(content => content.id === contentNode.id);
                        if (index != -1) {
                            //the item is already added
                            return;
                        }

                        var selectedContent = {
                            id: contentNode.id,
                            name: contentNode.name,
                            type: contentNode.type,
                            dimensionLevel: contentNode.type == "Dimension" ? contentNode.levelNumber : null,
                            selectConfig: contentNode.type == "Dimension" ? $scope.contentSettings.dimension[0] : null //sets Both as default value
                        }
                        $scope.selectedCubeContent.push(selectedContent);

                    } else {
                        //remove from selected content
                        var index = $scope.selectedCubeContent.findIndex(content => content.id === contentNode.id);
                        if (index > -1) {
                            $scope.selectedCubeContent.splice(index, 1);
                        }
                    }
                }

                //var checkcontentChildren; //TODO check IE for compatibility
                var checkContentChildren = function (contentNode) {
                    
                    updateSelectedContent(contentNode);

                    if (contentNode.contentChildren && contentNode.contentChildren.length > 0) {
                        contentNode.contentChildren.forEach(function (contentChildren) {
                            //nestedTable.setAllSelected(listNode.checked);
                            contentChildren.checked = contentNode.checked;
                            checkContentChildren(contentChildren);
                        });
                    }
                };

                //sets checked on all children
                checkContentChildren(contentNode);

                //console.log("selected content", $scope.selectedCubeContent);

                //$scope.onTableShow(list);
                //getScriptPreviewWithCheck();
            };


            $scope.onDataPreviewClick = function () {
                $scope.dataview = true;
                $scope.helpview = false;
                $scope.metadataview = false;

                //if (!$scope.preview || $scope.preview.length === 0) {
                //    loadDataPreview();
                //}
            };

            $scope.onContentConfigClick = function () {
                $scope.dataview = false;
                $scope.helpview = false;
                $scope.metadataview = true;
            };


            $scope.onHelpClick = function () {
                $scope.dataview = false;
                $scope.helpview = true;
                $scope.metadataview = false;
            };


            $scope.isAddScriptButtonEnabled = function () {

                if (!!$scope.selectedCubeContent && $scope.selectedCubeContent.length > 0) {

                    var indexMeasure = $scope.selectedCubeContent.findIndex(function (element) {
                        return element.type == "Measure";
                    });

                    if (indexMeasure != -1) {
                        var indexDimension = $scope.selectedCubeContent.findIndex(function (element) {
                            return element.type == "Dimension";
                        });

                        return indexDimension != -1;
                    }
                }

                return false;
            };


            //SEE Rest connector C:\Users\micha\AppData\Local\Programs\Common Files\Qlik\Custom Data\QvRestConnector\web\selectdialog.js
            $scope.onOK = function () {
                $scope.isLoading = true;

                var script = getCurrentScript($scope.selectedCubeContent, $scope.currentCube.id, $scope.currentCatalog.id, $scope.dimensionIncludeEmpty, $scope.measureIncludeEmpty)

                $scope.scriptEditorProxy.insertCode(script, $scope.dataconnectorInstance);
                $scope.destroyComponent();
            };

            $scope.onCancel = function () {
                $scope.cancelIsInProgress = true;
                $scope.destroyComponent();

                /* TODO - Implement this
			    engine.global.abortAll().then(function () {
			        if (scriptEditorProxy.cancelAction) {
			            scriptEditorProxy.cancelAction($scope.dataconnectorInstance).then(function () {
			                $scope.destroyComponent();
			            });
			        }
			        else {
			            $scope.destroyComponent();
			        }
			    });*/
            };

            //init model

            init();

        }]
    };
});



